/*
 * **********************************************************************************************
 * @file        gw_usb2spi.h
 * @author      EMB
 * @version     V1.0.0
 * @brief       Gowin USB to SPI driver and API
 ***********************************************************************************************
 */

#ifndef _GW_USB2SPI_H_
#define _GW_USB2SPI_H_


//Cross platform
#if defined(_WIN32) || defined(__CYGWIN__)
#ifndef DLLIMPORT
#define DLLIMPORT __declspec(dllexport)
#endif
#else
#define DLLIMPORT
#endif


/* Includes ---------------------------------------------------------------------- */
#include "libusb.h"
#include "channels.h"


#if defined(__cplusplus)
extern "C" {
#endif


/* Definitions ---------------------------------------------------------------------- */
#ifndef bool
#define bool unsigned char
#define true 1
#define TRUE 1
#define True 1

#define false 0
#define FALSE 0
#define False 0
#endif

#define DATA_PACKET_SIZE  128

// SPI transfer mode
#define SPI_TRANSMODE_WRnRD           (0x0)
#define SPI_TRANSMODE_WRONLY          (0x1)
#define SPI_TRANSMODE_RDONLY          (0x2)
#define SPI_TRANSMODE_WR_RD           (0x3)
#define SPI_TRANSMODE_RD_WR           (0x4)
#define SPI_TRANSMODE_WR_DMY_RD       (0x5)
#define SPI_TRANSMODE_RD_DMY_WR       (0x6)
#define SPI_TRANSMODE_NONEDATA        (0x7)
#define SPI_TRANSMODE_DMY_WR          (0x8)
#define SPI_TRANSMODE_DMY_RD          (0x9)

// SPI Data Format
#define SPI_DATAFORMAT_SINGLE  (0x0)
#define SPI_DATAFORMAT_DUAL    (0x1)
#define SPI_DATAFORMAT_QUAD    (0x2)

// SPI Addr Format
#define SPI_ADDRFORMAT_SINGLE  (0x0)
#define SPI_ADDRFORMAT_ASDATA  (0x1)

// SPI MOSI Dir, only available
#define SPI_MOSI_UNIDIR        (0x0)
#define SPI_MOIS_BIDIR         (0x1)

// SPI TokenValue
#define SPI_TOKENVALUE_0X00    (0x0)
#define SPI_TOKENVALUE_0X69    (0x1)

//#define Printf_Register
//USB to SPI configurations
typedef struct _USB2SPI_Config_
{
	unsigned char  CPha;
	unsigned char  CPol;
	unsigned char  LSB;
	unsigned char  DataMerge;
	unsigned char  DataLen;
	unsigned char  AddrLen;

	unsigned char  AddrEn;
	unsigned char  CmdEn;
	unsigned char  TransMode;
	unsigned char  DummyCnt;
	unsigned char  TokenEn;
	unsigned char  TokenValue;
	unsigned short RdTranCnt;
	unsigned short WrTranCnt;

	unsigned char  AddrFmt;    //Single/dual/quad
	unsigned char  DataFmt;

	unsigned char  SClkDiv;
	unsigned char  CSHT;
	unsigned char  CS2SClk;
} USB2SPI_Config;

#define USB2SPI_ERR_COUNT 20
/* Declarations ---------------------------------------------------------------------- */


DLLIMPORT const char * usb2spi_get_err_info(int errCode);
DLLIMPORT int usb2spi_print_config(USB2SPI_Config *pusb2spi_config);

DLLIMPORT int usb2spi_set_config(libusb_device_handle *devh, USB2SPI_Config *pusb2spi_config, unsigned int uiTimeout);
DLLIMPORT int usb2spi_get_config(libusb_device_handle *devh, USB2SPI_Config *pusb2spi_config, unsigned int uiTimeout);
DLLIMPORT int usb2spi_reset(libusb_device_handle *devh, bool bTxFifoRst, bool bRxFifoRst, bool bSpiRst, unsigned int uiTimeout);


// write only
// send data in bytes in merge mode, DataMerge should be set to 1 before this function, otherwise the data will not be transmit correctly.
DLLIMPORT int usb2spi_master_write_bytes_merge(libusb_device_handle *devh, USB2SPI_Config *pusb2spi_config,unsigned int uiCmd, unsigned int uiAddr, unsigned char *pucTxData, int iTxCnt, unsigned int uiTimeout);

// read only
// receive data in bytes in merge mode, DataMerge should be set to 1 before this function, otherwise the data will not be transmit correctly.
DLLIMPORT int usb2spi_master_read_bytes_merge(libusb_device_handle *devh,USB2SPI_Config *pusb2spi_config, unsigned int uiCmd, unsigned int uiAddr, unsigned char *pucRxData, int iRxCnt, unsigned int uiTimeout);

// write and read at the same time
// write and read data at the same time in bytes in merge mode, DataMerge should be set to 1 before this function, otherwise the data will not be transmit correctly.
DLLIMPORT int usb2spi_master_write_read_bytes_merge(libusb_device_handle *devh, USB2SPI_Config *pusb2spi_config,unsigned int uiCmd, unsigned int uiAddr, unsigned char *pucTxData, unsigned char *pucRxData, int iTxCnt, int iRxCnt, unsigned int uiTimeout);


#if defined(__cplusplus)
}
#endif


#endif   //#ifndef _GW_USB2SPI_H_
