/*
 * *****************************************************************************************
 * @file		gw_usb2uart.h
 * @author		EMB
 * @version		V1.0.0
 * @brief		Gowin USB2UART API.
 ******************************************************************************************
 */

#ifndef GW_USB2UART_H_
#define GW_USB2UART_H_


//Cross platform
#if defined(_WIN32) || defined(__CYGWIN__)
#ifndef DLLIMPORT
#define DLLIMPORT __declspec(dllexport)
#endif
#else
#define DLLIMPORT
#endif


/* Includes ------------------------------------------------------------------------------------------------------------- */
#include "libusb.h"
#include "channels.h"

#if defined(__cplusplus)
extern "C" {
#endif


/* Definitions ------------------------------------------------------------------------------------------------------------- */
//UART LIBUSB
//Stop bits
#define STOPBITS_1    0			//1
#define STOPBITS_1_5  1			//1.5
#define STOPBITS_2    2			//2

//Parity
#define GW_PARITY_NONE   0		//none
#define GW_PARITY_ODD    1		//odd
#define GW_PARITY_EVEN   2		//even
#define GW_PARITY_MARK   3
#define GW_PARITY_SPACE  4

#define PACK_SIZE   64

//USB2UART configurations
typedef struct _usb2uart_config_setting {
	unsigned int  BaudRate;		//baud rate
	unsigned char StopBits;		//stop bits
	unsigned char Parity;		    //parity
	unsigned char DataBits;		//data bits
} usb2uart_config_setting;

/* Declarations ------------------------------------------------------------------------------------------------------------- */
//UART LIBUSB
//Set USB2UART configurations
DLLIMPORT int usb2uart_set_config(libusb_device_handle *devh, usb2uart_config_setting* pusb2uart_config, unsigned char interface, unsigned int uiTimeout);

//Print USB2UART configurations
DLLIMPORT int print_usb2uart_config(usb2uart_config_setting* pusb2uart_config);

//USB2UART send data synchronously
DLLIMPORT int usb2uart_send_data(libusb_device_handle *devh, unsigned char endpoint_out, int DataCnt, unsigned char *TransData,  unsigned int uiTimeout);

//USB2UART receive data synchronously
DLLIMPORT int usbuart_receive_data(libusb_device_handle *devh, unsigned char endpoint_in, int *DataCnt, unsigned char* TransData, unsigned int uiTimeout);


/* Definitions ------------------------------------------------------------------- */
//UART VCP
//Open UART
int UART0_Open(char* port);

//Close UART
void UART0_Close(int fd);

//Initializes UART
int UART0_Init(int fd, int speed, int flow_control, int databits, int stopbits, int parity);

//Set UART configurations
int UART0_Set(int fd, int speed, int flow_ctrl, int databits, int stopbits, int parity);

//UART receives data
int UART0_Recv(int fd, char *rcv_buf, int data_len);

//UART sends data
int UART0_Send(int fd, char *send_buf, int data_len);



#if defined(__cplusplus)
}
#endif

#endif // GW_USB2UART_H_
