
/*
 * **********************************************************************************************
 * @file        gwusb_demo.c
 * @author      EMB
 * @version     V1.0.0
 * @brief       Demo
 ***********************************************************************************************
 */

/* Includes ------------------------------------------------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "libusb.h"
#include "gw_usb2iic.h"
#include "gw_usb2uart.h"
#include "gw_usb2spi.h"
#include "gw_usb2jtag.h"
#include "user_define.h"


/* Definitions ------------------------------------------------------------------------------------------------------------- */

//Libusb device handler
static struct libusb_device_handle *devh = NULL;
const char filename[50]="fpcs_demo.bin";

//----------- USB to UART----------//
#if USB2UART_TEST
unsigned char tx_data[] = "this is a usb2uart test string sent through usb\r\n";
unsigned int rd_cnt = 0;
usb2uart_config_setting usb2uart_config;
#endif//USB2UART_TEST

#if USB2UART_VCP_TEST
char rcv_buf[100];                                                              //receive data
char send_buf[100] = "Gowin : virtual com port usb2uart test test test...";     //send data

//uart serial port
char uart_port[] = "/dev/ttyUSB0";      //ls -l /dev/ttyUSB*
#endif//USB2UART_VCP_TEST

//----------- USB to I2C ----------//
#if USB2IIC_TEST
static USB2IIC_Config  usb2iic_config;
USB2IIC_Config *pusb2iic_config = &usb2iic_config;

unsigned char WData[20] = {1,2,3,4,5,6,7,8,9,10,11,12, 13, 14, 15, 16, 17, 18, 19, 20};
unsigned char RData[256];
#endif // USB2IIC_TEST

//------------ USB to SPI ----------//
#if USB2SPI_TEST
static USB2SPI_Config usb2spi_config;
USB2SPI_Config *pusb2spi_config = &usb2spi_config;

#define TEST_SIZE 16
#define RX_SIZE   10
#endif // USB2SPI_TEST

#if USB2JTAG_TEST
    int length =3;
#endif // USB2JTAG_TEST

/* Declarations --------------------------------------------------------------------------------------------------------- */
//------------ USB to I2C ----------//
#if USB2IIC_TEST
int usb2iic_test(libusb_device_handle *devh, USB2IIC_Config *pusb2iic_config);
#if I2C_CONFIG_SETTING_MODE
int iic_config_set_test(libusb_device_handle *devh,  USB2IIC_Config *pusb2iic_config);
#endif // I2C_CONFIG_SETTING_MODE
#if I2C_MASTER_WRITE_MODE
int iic_master_write_test(libusb_device_handle *devh, USB2IIC_Config *pusb2iic_config);
#endif // I2C_MASTER_WRITE_MODE
#if I2C_MASTER_READ_MODE
int iic_master_read_test(libusb_device_handle *devh,  USB2IIC_Config *pusb2iic_config);
#endif // I2C_MASTER_READ_MODE
#if I2C_SLAVE_WRITE_MODE
int iic_slave_write_test(libusb_device_handle *devh, USB2IIC_Config *pusb2iic_config);
#endif // I2C_SLAVE_WRITE_MODE
#if I2C_SLAVE_READ_MODE
int iic_slave_read_test(libusb_device_handle *devh, USB2IIC_Config *pusb2iic_config);
#endif // I2C_SLAVE_READ_MODE
#endif // USB2IIC_TEST

//------------ USB to SPI -----------//
#if USB2SPI_TEST
int usb2spi_test(libusb_device_handle *devh, USB2SPI_Config *pusb2spi_config);
#if SPI_WRITE_MODE
int usb2spi_write_test(libusb_device_handle *devh,USB2SPI_Config *pusb2spi_config, unsigned int uiTimeout);
#endif // SPI_WRITE_MODE
#if SPI_READ_MODE
int usb2spi_read_test(libusb_device_handle *devh, USB2SPI_Config *pusb2spi_config, unsigned int uiTimeout);
#endif // SPI_READ_MODE
#if SPI_WRITE_AND_READ_MODE
int usb2spi_write_read_test(libusb_device_handle *devh, USB2SPI_Config *pusb2spi_config, unsigned int uiTimeout);
#endif // SPI_WRITE_AND_READ_MODE
#endif // USB2SPI_TEST

#if USB2UART_TEST
int usb2uart_test(void);
#endif // USB2UART_TEST

#if USB2UART_VCP_TEST
int usb2uart_vcp_test(void);
#endif // USB2UART_VCP_TEST


/* Functions ------------------------------------------------------------------------------------------------------------- */
// ---------------------------------------- USB to UART---------------------------------------//
#if USB2UART_TEST

/* Functions ------------------------------------------------------------------------------------------------------------- */
//Call back funtions of asynchronous received, output to console by string
void LIBUSB_CALL cb_xfr_in(struct libusb_transfer *xfr)
{
	int i;

	rd_cnt++;

	if (xfr->status != LIBUSB_TRANSFER_COMPLETED)
    {
		fprintf(stderr, "transfer status %d\n", xfr->status);
		libusb_free_transfer(xfr);
		exit(3);
	}

	for (i = 0; i < xfr->actual_length; i++)
    {
		printf("%c", xfr->buffer[i]);
	}

     //After received data this time, request to transmit again, and wait next event
	if(libusb_submit_transfer(xfr) < 0)
	{
		libusb_free_transfer(xfr);
	}
}

static int benchmark_in(uint8_t ep)
{
	static uint8_t buf[PACK_SIZE];
	static struct libusb_transfer *xfr;

	printf("(SUBMIT FUNC) Starting a new asynchronous bulk transfer\n");
	printf("Received data :\n");

	xfr = libusb_alloc_transfer(0);
	if (!xfr)
    {
		return -ENOMEM;
	}

	libusb_fill_bulk_transfer (
		xfr,
		devh,
		ep,
		buf,
		sizeof(buf),
		cb_xfr_in,
		NULL,
		0
	);

	return libusb_submit_transfer(xfr);
}

int usb2uart_test()
{
	int rc = 0;

    rc = libusb_claim_interface(devh, UART_INTERFACE);
	if (rc != LIBUSB_SUCCESS)
    {
		printf("Error claiming interface: %s\n", libusb_error_name(rc));
		goto out;
	}
	else
    {
        printf("Claiming interface\n");
    }

	printf("DEVICE READY...\n\n");
	//Configurations settings
	usb2uart_config.BaudRate  = 115200;
	usb2uart_config.Parity    = GW_PARITY_NONE;
	usb2uart_config.StopBits  = STOPBITS_1;
	usb2uart_config.DataBits  = 8;

	//Set configurations
	rc = usb2uart_set_config(devh, &usb2uart_config, UART_INTERFACE, 0);
	if(rc != 0)
	{
		printf("Error: %s", libusb_strerror(rc));
		return -1;
	}

    //Send data synchronously
	rc = usb2uart_send_data(devh, UART_USB_EP_OUT, sizeof(tx_data), tx_data,  1000);
	if(rc != 0)
    {
		printf("Error: %s", libusb_strerror(rc));
	}
	else
	{
		printf("Send %d bytes, return code : %d\n", (int)(sizeof(tx_data)), rc);
	}

	benchmark_in(UART_USB_EP_IN);

	while(1)
    {
		rc = libusb_handle_events(NULL);
		//usbuart_receive_data(devh, UART_USB_EP_IN, DataCnt, TransData, 500);
	}

	libusb_release_interface(devh, UART_INTERFACE);
out:
	if(devh)
    {
		libusb_close(devh);
    }

	libusb_exit(NULL);
return 0;
}
//Call back functions of asynchronous received, output to console by string

#endif // USB2UART_TEST
#if USB2UART_VCP_TEST
int usb2uart_vcp_test()
{
    int fd = 0;
    int err;
    int len;
    int i = 0;

    //open uart
    fd = UART0_Open(uart_port);
    if(FALSE == fd)
    {
        return fd;
    }

    do
    {
        //initializes uart
        //set uart
        //baudrate  : 115200
        //flow ctrl : rts/cts
        //databit   : 8
        //stopbit   : 1
        //parity    : none
        err = UART0_Init(fd, 115200, 0, 8, 1, 'N');
    }while(-1 == err || -1 == fd);

    printf("uart init complete...\n");

    // uart send test
    len = UART0_Send(fd, send_buf, 51);
    if(len > 0)
    {
        printf("send data success...\n");
    }

    while(i < 1000)
    {
        len = UART0_Recv(fd, rcv_buf, 99);

        if(len >0)
        {
            rcv_buf[len] = '\0';
            printf("%s", rcv_buf);
            len = 0;
        }

        i++;
    }

    if(i == 1000)
    {
        printf("\nwait to receive data timeout, returned.\n");
    }

    if(fd != -1)
    {
        UART0_Close(fd);
    }
return 0;
}
#endif // USB2UART_VCP_TEST


// ---------------------------------------- USB to SPI ----------------------------------------//
#if USB2SPI_TEST

//Test USB to SPI
int usb2spi_test(libusb_device_handle *devh, USB2SPI_Config *pusb2spi_config)
{
    int rc = 0;


    //Check parameters
    if((devh == NULL) || (pusb2spi_config == NULL))
    {
        printf("USB to SPI parameters error.\n");
        return USB_ERROR_INVALID_PARAM;
    }

    //Claim interface
    rc = libusb_claim_interface(devh, Interface_number);
	if (rc != SUCCESS)
	{
		printf("Error claiming interface %d : %s\n", Interface_number, libusb_error_name(rc));
		return rc;
	}
	else
    {
        printf("Claiming interface successfully.\n");
    }

    printf("\nDEVICE READY...\n\n");

#if SPI_WRITE_MODE
    //Test USB to SPI write mode
    rc = usb2spi_write_test(devh, pusb2spi_config, 1000);
    if(rc != SUCCESS)
    {
        goto out;
    }
#endif // SPI_WRITE_MODE

#if SPI_READ_MODE
    //Test USB to SPI read mode
	rc = usb2spi_read_test(devh, pusb2spi_config, 1000);
	if(rc != SUCCESS)
    {
        goto out;
    }
#endif // SPI_READ_MODE

#if SPI_WRITE_AND_READ_MODE
    //Test USB to SPI read write mode
	rc = usb2spi_write_read_test(devh, pusb2spi_config, 1000);
	if(rc != SUCCESS)
    {
        goto out;
    }
#endif // SPI_READ_MODE


out:
	libusb_release_interface(devh, Interface_number);

    return rc;
}

#if SPI_WRITE_MODE
//Test USB to SPI write data
int usb2spi_write_test(libusb_device_handle *devh,USB2SPI_Config *pusb2spi_config, unsigned int uiTimeout)
{
	int rc = 0;
	unsigned char data[TEST_SIZE];
	int i = 0;

	//Check parameter
	if((devh == NULL) || (pusb2spi_config == NULL))
    {
        printf("USB to SPI parameters error.\n");
        return USB_ERROR_INVALID_PARAM;
    }

	//SPI configurations
	pusb2spi_config->CS2SClk = 1;
	pusb2spi_config->SClkDiv = 0;
	pusb2spi_config->CSHT    = 1;
	pusb2spi_config->CmdEn   = 1;
	pusb2spi_config->AddrEn  = 1;
	pusb2spi_config->WrTranCnt = TEST_SIZE-1;
	pusb2spi_config->RdTranCnt = RX_SIZE-1;
	pusb2spi_config->CPha = 0;
	pusb2spi_config->CPol = 0;
	pusb2spi_config->LSB  = 0;
	pusb2spi_config->AddrLen = 3;
	pusb2spi_config->DataLen = 7;
	pusb2spi_config->DataMerge = 1;

	pusb2spi_config->DummyCnt = 0;
	pusb2spi_config->TransMode = SPI_TRANSMODE_WRONLY;
	pusb2spi_config->TokenEn = FALSE;
	pusb2spi_config->TokenValue = SPI_TOKENVALUE_0X00;

	pusb2spi_config->DataFmt = SPI_DATAFORMAT_SINGLE;
	pusb2spi_config->AddrFmt = SPI_ADDRFORMAT_SINGLE;

    for(i = 0; i < TEST_SIZE; i++)
	{
		data[i] = (i + 1) * 0x11;
	}

	//Write bytes data in merge mode
	rc = usb2spi_master_write_bytes_merge(devh,pusb2spi_config, 0x02, 0x00000000, (unsigned char *)data, TEST_SIZE, uiTimeout);
	if(rc != SUCCESS)
	{
		printf("Send USB to SPI master bytes merge failed : %s\n",libusb_error_name(rc));
		return rc;
	}
	else
	{
		printf("Send USB to SPI master bytes merge successfully.\n");
	}

	sleep(400);

	printf("USB to SPI master write data: \n");

	for(i = 0; i < TEST_SIZE; i++)
	{
		printf("0x%02x ", data[i]);
		if((i + 1) % 16 == 0)
		{
			putchar('\n');
		}
		else if((i + 1) % 4 == 0)
		{
			putchar('\t');
		}
	}
	putchar('\n');

    //Get USB to SPI configurations
	rc = usb2spi_get_config(devh,pusb2spi_config, uiTimeout);
	if(rc != SUCCESS)
	{
		return rc;
	}

	return SUCCESS;
}
#endif // SPI_WRITE_MODE


//Test USB to SPI read data
#if SPI_READ_MODE
int usb2spi_read_test(libusb_device_handle *devh, USB2SPI_Config *pusb2spi_config, unsigned int uiTimeout)
{
	int rc = 0;
	unsigned char data[TEST_SIZE];
	int i = 0;

	//Check parameters
	if((devh == NULL) || (pusb2spi_config == NULL))
    {
        printf("USB to SPI parameters error.\n");
        return USB_ERROR_INVALID_PARAM;
    }

	for(i = 0; i < TEST_SIZE; i++)
	{
		data[i] = i;
	}

	//SPI configurations
	pusb2spi_config->CS2SClk = 1;
	pusb2spi_config->SClkDiv = 0;
	pusb2spi_config->CSHT    = 1;
	pusb2spi_config->CmdEn   = 0;
	pusb2spi_config->AddrEn  = 0;
	pusb2spi_config->WrTranCnt = TEST_SIZE-1;
	pusb2spi_config->RdTranCnt = RX_SIZE-1;
	pusb2spi_config->CPha = 0;
	pusb2spi_config->CPol = 0;
	pusb2spi_config->LSB  = 0;
	pusb2spi_config->AddrLen = 3;
	pusb2spi_config->DataLen = 7;
	pusb2spi_config->DataMerge = 1;
	pusb2spi_config->TransMode = SPI_TRANSMODE_RDONLY;

	pusb2spi_config->DummyCnt = 0;
	pusb2spi_config->TokenEn = FALSE;
	pusb2spi_config->TokenValue = SPI_TOKENVALUE_0X00;

	pusb2spi_config->DataFmt = SPI_DATAFORMAT_SINGLE;
	pusb2spi_config->AddrFmt = SPI_ADDRFORMAT_SINGLE;


	printf("Read USB to SPI test...\n");

	//Read USB to SPI bytes data
	rc = usb2spi_master_read_bytes_merge(devh,pusb2spi_config, 0x03, 0x00000000, data, RX_SIZE, uiTimeout);
	if(rc != SUCCESS)
	{
		printf("Read USB to SPI master bytes failed : %s\n", libusb_error_name(rc));
		return rc;
	}
	else
	{
		printf("Read USB to SPI master bytes successfully.\n");
	}

	printf("Received data:\n");

	for(i = 0; i < RX_SIZE; i++)
	{
		printf("0x%02x ", data[i]);
		if((i + 1) % 8 == 0)
		{
			putchar('\n');
		}
		else if((i + 1) % 4 == 0)
		{
			putchar(' ');
		}
	}
	putchar('\n');

	//Get USB to SPI configurations
	rc = usb2spi_get_config(devh,pusb2spi_config, uiTimeout);
	if(rc != SUCCESS)
	{
		return rc;
	}

	return SUCCESS;
}
#endif // SPI_READ_MODE

#if SPI_WRITE_AND_READ_MODE

int usb2spi_write_read_test(libusb_device_handle *devh, USB2SPI_Config *pusb2spi_config, unsigned int uiTimeout)
{
	int rc = 0;
	//unsigned int  idata[TEST_SIZE];
	unsigned char data[TEST_SIZE];

	//unsigned int  iRxData[TEST_SIZE];
	unsigned char cRxData[RX_SIZE];

	int i = 0;


	//SPI configurations
	pusb2spi_config->CS2SClk = 1;
	pusb2spi_config->SClkDiv = 0;
	pusb2spi_config->CSHT    = 1;
	pusb2spi_config->CmdEn   = 0;
	pusb2spi_config->AddrEn  = 0;
	pusb2spi_config->WrTranCnt = TEST_SIZE-1;
	pusb2spi_config->RdTranCnt = RX_SIZE-1;
	pusb2spi_config->CPha = 0;
	pusb2spi_config->CPol = 0;
	pusb2spi_config->LSB  = 0;
	pusb2spi_config->AddrLen = 3;
	pusb2spi_config->DataLen = 7;
	pusb2spi_config->DataMerge = 1;
	pusb2spi_config->TransMode = SPI_TRANSMODE_WR_RD;
	//pusb2spi_config->TransMode = SPI_TRANSMODE_WRnRD;//  SPI_TRANSMODE_WR_RD;

	pusb2spi_config->DummyCnt = 0;
	pusb2spi_config->TokenEn = FALSE;
	pusb2spi_config->TokenValue = SPI_TOKENVALUE_0X69;

	pusb2spi_config->DataFmt = SPI_DATAFORMAT_SINGLE;
	pusb2spi_config->AddrFmt = SPI_ADDRFORMAT_SINGLE;


	for(i = 0; i < TEST_SIZE; i++)
	{
		data[i] = (i + 1)*0x11;
	}

	rc = usb2spi_master_write_read_bytes_merge(devh, pusb2spi_config,0xab, 0xcdcdcdcd, (unsigned char *)data, cRxData, TEST_SIZE, RX_SIZE, 1000);
	if(rc != SUCCESS)
	{
		printf("usb2spi master write and read bytes merge failed : %s\n", libusb_error_name(rc));
		return rc;
	}
	else
	{
		printf("usb2spi master write and read bytes merge successfully.\n");
	}

	printf("Received data:\n");
	for(i = 0; i < RX_SIZE; i++)
	{
		printf("0x%02x ", cRxData[i]);
		if((i + 1) % 16 == 0)
		{
			putchar('\n');
		}
		else if((i + 1) % 4 == 0)
		{
			putchar(' ');
		}
	}
	putchar('\n');



	return SUCCESS;
}
#endif // SPI_WRITE_AND_READ_MODE

#endif // USB2SPI_TEST


// ---------------------------------------- USB to I2C ----------------------------------------//
#if USB2IIC_TEST

//Test USB to I2C
int usb2iic_test(libusb_device_handle *devh, USB2IIC_Config *pusb2iic_config)
{
    int rc = 0;


    //Check parameters
    if((devh == NULL) || (pusb2iic_config == NULL))
    {
        printf("USB to I2C parameters error.\n");
        return USB_ERROR_INVALID_PARAM;
    }

    //Claim interface
    rc = libusb_claim_interface(devh, Interface_number);
	if (rc != SUCCESS)
	{
		printf("Error claiming interface %d : %s\n", Interface_number, libusb_error_name(rc));
		return rc;
	}
	else
    {
        printf("Claiming interface successfully.\n");
    }

    printf("\nDEVICE READY...\n\n");

#if I2C_CONFIG_SETTING_MODE
    //Test configurations setting
    rc = iic_config_set_test(devh, pusb2iic_config);
    if(rc != SUCCESS)
    {
        goto out;
    }
#endif // I2C_CONFIG_SETTING_MODE

#if I2C_MASTER_WRITE_MODE
    //Run USB2IIC, then PC read data in Software "USB2XXX"(test box)
	//Test box is slave
	rc = iic_master_write_test(devh, pusb2iic_config);
	if(rc != SUCCESS)
    {
        goto out;
    }
#endif // I2C_MASTER_WRITE_MODE

#if I2C_MASTER_READ_MODE
	//PC write data in Software "USB2XXX"(test box), then run USB2IIC
	//Test box is slave
    rc = iic_master_read_test(devh,pusb2iic_config);
    if(rc != SUCCESS)
    {
        goto out;
    }
#endif // I2C_MASTER_READ_MODE

#if I2C_SLAVE_WRITE_MODE
	//Run USB2IIC, then PC read data in Software "USB2XXX"(test box)
	//Test box is master
	rc = iic_slave_write_test(devh,pusb2iic_config);
	if(rc != SUCCESS)
    {
        goto out;
    }
#endif // I2C_SLAVE_WRITE_MODE

#if I2C_SLAVE_READ_MODE
	//Run USB2IIC, then PC write data in Software "USB2XXX"(test box)
	//Test box is master
	rc = iic_slave_read_test(devh, pusb2iic_config);
	if(rc != SUCCESS)
    {
        goto out;
    }
#endif // I2C_SLAVE_READ_MODE

out:
	libusb_release_interface(devh, Interface_number);

    return rc;
}


#if I2C_CONFIG_SETTING_MODE
//Test USB to IIC configurations setting
int iic_config_set_test(libusb_device_handle *devh,  USB2IIC_Config *pusb2iic_config)
{
    int rc = 0;

    //Check parameters
    if((devh == NULL) || (pusb2iic_config == NULL))
    {
        printf("USB to I2C parameters error.\n");
        return USB_ERROR_INVALID_PARAM;
    }

    //Get I2C configurations
    rc = iic_get_config(devh,pusb2iic_config, 1000);
    if(rc != SUCCESS)
    {
        return rc;
    }
	printf("\nDefault I2C configuration settings: \n");

	rc = print_iic_config(pusb2iic_config);
	if(rc != SUCCESS)
    {
        return rc;
    }

	//Standard mode, 100KHz
#if I2C_MODE_100KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 8;
	pusb2iic_config->HoldDat = 11;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 55;
	pusb2iic_config->TParam = 3;
#endif

	//Fast mode: 400KHz
#if I2C_MODE_400KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 5;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 54;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 1000KHz
#if I2C_MODE_1000KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 18;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 2400KHz
#if I2C_MODE_2400KHZ
	pusb2iic_config->SpikePulse = 2;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 6;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 2600KHz
#if I2C_MODE_2600KHZ
	pusb2iic_config->SpikePulse = 2;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 5;
	pusb2iic_config->TParam = 0;
#endif

	pusb2iic_config->Addr = 0x50;
	pusb2iic_config->PhaseStart = 1;
	pusb2iic_config->PhaseAddr = 1;
	pusb2iic_config->PhaseData = 1;
	pusb2iic_config->PhaseStop = 1;
	pusb2iic_config->Master = 1;
	pusb2iic_config->Addressing = 0;
	pusb2iic_config->IICEnable = 1;

	printf("\nI2C configuration settins...\n");

	rc = print_iic_config(pusb2iic_config);
	if(rc != SUCCESS)
    {
        return rc;
    }

	rc = iic_set_config(devh, pusb2iic_config, 1000);
	if(rc != SUCCESS)
    {
        return rc;
    }

	rc = iic_get_config(devh,pusb2iic_config, 1000);
	if(rc != SUCCESS)
    {
        return rc;
    }

	printf("\nNew I2C configuration settings: \n");

	rc = print_iic_config(pusb2iic_config);
	if(rc != SUCCESS)
    {
        return rc;
    }

    return SUCCESS;
}
#endif // I2C_CONFIG_SETTING_MODE


#if I2C_MASTER_WRITE_MODE
//Test USB to IIC as master, write data to slave
int iic_master_write_test(libusb_device_handle *devh,  USB2IIC_Config *pusb2iic_config)
{
    int rc = 0;

    //Check parameters
    if((devh == NULL) || (pusb2iic_config == NULL))
    {
        printf("USB to I2C parameters error.\n");
        return USB_ERROR_INVALID_PARAM;
    }

	//Set iic configurations
	//Standard mode, 100KHz
#if I2C_MODE_100KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 8;
	pusb2iic_config->HoldDat = 11;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 55;
	pusb2iic_config->TParam = 3;
#endif

	//Fast mode: 400KHz
#if I2C_MODE_400KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 5;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 54;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 1000KHz
#if I2C_MODE_1000KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 18;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 2400KHz
#if I2C_MODE_2400KHZ
	pusb2iic_config->SpikePulse = 2;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 6;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 2600KHz
#if I2C_MODE_2600KHZ
	pusb2iic_config->SpikePulse = 2;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 5;
	pusb2iic_config->TParam = 0;
#endif

	pusb2iic_config->Addr = 0x50;
	pusb2iic_config->PhaseStart = 1;
	pusb2iic_config->PhaseAddr = 1;
	pusb2iic_config->PhaseData = 1;
	pusb2iic_config->PhaseStop = 1;
	pusb2iic_config->Master = 1;
	pusb2iic_config->Addressing = 0;
	pusb2iic_config->IICEnable = 1;

	rc = iic_set_config(devh,pusb2iic_config, 1000);
	if(rc != SUCCESS)
    {
        printf("Setting USB to I2C master configurations failed.\n");
        return rc;
    }
    else
    {
        printf("Setting USB to I2C master configurations successfully.\n");
    }

	printf("USB to I2C master write demo, send %lu bytes to I2C slave device\n", (long unsigned int)(sizeof(WData)));
	rc = usb2iic_master_write(devh, sizeof(WData), WData, 5000);
	if(rc != SUCCESS)
    {
		printf("USB to I2C master write data failed.\n");
		return rc;
	}
	else
    {
        printf("USB to I2C master write data successfully.\n");
    }

	rc = iic_get_config(devh, pusb2iic_config, 1000);
	if(rc != SUCCESS)
	{
        printf("USB to I2C master get configurations failed.\n");
        return rc;
    }
    else
    {
        printf("USB to I2C master get configurations successfully.\n");
    }

	printf("\nUSB to I2C configurations after master write: \n");

	rc = print_iic_config(pusb2iic_config);
	if(rc != SUCCESS)
    {
        return rc;
    }

    return SUCCESS;
}
#endif // I2C_MASTER_WRITE_MODE


#if I2C_MASTER_READ_MODE
//Test USB to IIC as master, read data from slave
int iic_master_read_test(libusb_device_handle *devh, USB2IIC_Config *pusb2iic_config)
{
    int rc = 0;
	int i = 0;

	//Check parameters
    if((devh == NULL) || (pusb2iic_config == NULL))
    {
        printf("USB to I2C parameters error.\n");
        return USB_ERROR_INVALID_PARAM;
    }

	//Standard mode, 100KHz
#if I2C_MODE_100KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 8;
	pusb2iic_config->HoldDat = 11;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 55;
	pusb2iic_config->TParam = 3;
#endif

	//Fast mode: 400KHz
#if I2C_MODE_400KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 5;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 54;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 1000KHz
#if I2C_MODE_1000KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 18;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 2400KHz
#if I2C_MODE_2400KHZ
	pusb2iic_config->SpikePulse = 2;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 6;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 2600KHz
#if I2C_MODE_2600KHZ
	pusb2iic_config->SpikePulse = 2;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 5;
	pusb2iic_config->TParam = 0;
#endif

	pusb2iic_config->Addr = 0x50;
	pusb2iic_config->PhaseStart = 1;
	pusb2iic_config->PhaseAddr = 1;
	pusb2iic_config->PhaseData = 1;
	pusb2iic_config->PhaseStop = 1;
	pusb2iic_config->Master = 1;
	pusb2iic_config->Addressing = 0;
	pusb2iic_config->IICEnable = 1;

	rc = iic_set_config(devh, pusb2iic_config, 1000);
	if(rc != SUCCESS)
    {
        return rc;
    }

	printf("USB to I2C master read demo:\n");

	rc = usb2iic_master_read(devh, 9, RData, 0);
	if(rc != SUCCESS)
    {
		printf("USB to I2C master read data failed.\n");
		return rc;
	}

	printf("USB to I2C master read data:\n");
	for(i = 0; i < 9; i++)
	{
		printf("%02x ", RData[i]);
		if((i + 1) % 4 == 0)
        {
			printf(" ");
		}
	}
	printf("\n");

    return SUCCESS;
}
#endif // I2C_MASTER_READ_MODE


#if I2C_SLAVE_WRITE_MODE
//Test USB to IIC as slave, write data to slave
int iic_slave_write_test(libusb_device_handle *devh,  USB2IIC_Config *pusb2iic_config)
{
    unsigned char TransData[256];
	int DataCnt = 8;

	int dataTransed = 0;

	int i = 0;
	int rc = 0;

	//Check parameters
    if((devh == NULL) || (pusb2iic_config == NULL))
    {
        printf("USB to I2C parameters error.\n");
        return USB_ERROR_INVALID_PARAM;
    }

	for(i = 0; i < DataCnt; i++)
    {
		TransData[i] = (i + 1) * 2;
	}

	//Standard mode, 100KHz
#if I2C_MODE_100KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 8;
	pusb2iic_config->HoldDat = 11;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 55;
	pusb2iic_config->TParam = 3;
#endif

	//Fast mode: 400KHz
#if I2C_MODE_400KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 5;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 54;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 1000KHz
#if I2C_MODE_1000KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 18;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 2400KHz
#if I2C_MODE_2400KHZ
	pusb2iic_config->SpikePulse = 2;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 6;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 2600KHz
#if I2C_MODE_2600KHZ
	pusb2iic_config->SpikePulse = 2;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 5;
	pusb2iic_config->TParam = 0;
#endif

	pusb2iic_config->Addr = 0x50;
	pusb2iic_config->PhaseStart = 1;
	pusb2iic_config->PhaseAddr = 1;
	pusb2iic_config->PhaseData = 1;
	pusb2iic_config->PhaseStop = 1;
	pusb2iic_config->Master = 0;
	pusb2iic_config->Addressing = 0;
	pusb2iic_config->IICEnable = 1;

	rc = iic_set_config(devh, pusb2iic_config, 1000);
	if(rc != SUCCESS)
    {
        return rc;
    }

	rc = usb2iic_slave_write(devh,  DataCnt, TransData, &dataTransed, 0);
	if(rc != SUCCESS)
    {
		printf("USB to I2C slave write test failed.\n");
		return rc;
	}

	printf("USB to I2C slave send bytes: %d\n", dataTransed);

    return SUCCESS;
}
#endif // I2C_SLAVE_WRITE_MODE


#if I2C_SLAVE_READ_MODE
//Test USB to IIC as slave, read data from slave
int iic_slave_read_test(libusb_device_handle *devh,  USB2IIC_Config *pusb2iic_config)
{
    unsigned char RcvData[256];
	int DataCnt;

	int i = 0;
	int rc = 0;

	//Check parameters
    if((devh == NULL) || (pusb2iic_config == NULL))
    {
        printf("USB to I2C parameters error.\n");
        return USB_ERROR_INVALID_PARAM;
    }

	//Standard mode, 100KHz
#if I2C_MODE_100KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 8;
	pusb2iic_config->HoldDat = 11;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 55;
	pusb2iic_config->TParam = 3;
#endif

	//Fast mode: 400KHz
#if I2C_MODE_400KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 5;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 54;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 1000KHz
#if I2C_MODE_1000KHZ
	pusb2iic_config->SpikePulse = 3;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 18;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 2400KHz
#if I2C_MODE_2400KHZ
	pusb2iic_config->SpikePulse = 2;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 6;
	pusb2iic_config->TParam = 0;
#endif

	//Fast plus mode: 2600KHz
#if I2C_MODE_2600KHZ
	pusb2iic_config->SpikePulse = 2;
	pusb2iic_config->SetupDat  = 2;
	pusb2iic_config->HoldDat = 2;
	pusb2iic_config->SclRatio = 0;
	pusb2iic_config->SclHi = 5;
	pusb2iic_config->TParam = 0;
#endif

	pusb2iic_config->Addr = 0x50;
	pusb2iic_config->PhaseStart = 1;
	pusb2iic_config->PhaseAddr = 1;
	pusb2iic_config->PhaseData = 1;
	pusb2iic_config->PhaseStop = 1;
	pusb2iic_config->Master = 0;
	pusb2iic_config->Addressing = 0;
	pusb2iic_config->IICEnable = 1;

	rc = usb2iic_reset(devh, 1000);
	if(rc != SUCCESS)
    {
        return rc;
    }

	rc = iic_set_config(devh,pusb2iic_config, 1000);
	if(rc != SUCCESS)
    {
        return rc;
    }

	rc = usb2iic_slave_read(devh, RcvData, &DataCnt, 0);
	if(rc != SUCCESS)
    {
		printf("USB to I2C slave read test failed.\n");
		return rc;
	}

	printf("USB to I2C read data: %d bytes received\n", DataCnt);
	for(i = 0; i < DataCnt; i++)
	{
		printf("%02x ", RcvData[i]);
		if((i + 1) % 4 == 0)
        {
			printf("  ");
		}
		else if((i + 1) % 32 == 0)
		{
			printf("\n");
		}
	}
	printf("\n");

    return SUCCESS;
}
#endif // I2C_SLAVE_READ_MODE

#endif // USB2IIC_TEST

// ---------------------------------------- USB to I2C ----------------------------------------//

// ---------------------------------------- USB to JTAG ----------------------------------------//
#if USB2JTAG_TEST
int usb2jtag_test(libusb_device_handle *devh, unsigned int uiTimeout)
{
    int rc = 0;

    //Check parameters
    if(devh == NULL)
    {
        printf("USB to JTAG parameters error.\n");
        return USB_ERROR_INVALID_PARAM;
    }

    //Claim interface
    rc = libusb_claim_interface(devh, Interface_number);
	if (rc != SUCCESS)
	{
		printf("Error claiming interface %d : %s\n", Interface_number, libusb_error_name(rc));
		return rc;
	}
	else
    {
        printf("Claiming interface successfully.\n");
    }

    printf("\nDEVICE READY...\n\n");

#if USB2JTAG_SELECT_TCK

    //Test USB to JTAG select tck
    printf("Select TCK TEST\r\n\r\n");
    rc=usb2jtag_tck_select(devh,Select_Tck,uiTimeout);
    if(rc != SUCCESS)
    {
        goto out;
    }

#endif // USB2JTAG_SELECT_TCK

#if JTAG_READ_IDCODE

    //Test USB to JTAG read idcode
    printf("Read IDCODE TEST\r\n\r\n");
    rc = usb2jtag_Read_IDCODE(devh,uiTimeout);
    if(rc != SUCCESS)
    {
        goto out;
    }

#endif // JTAG_READ_IDCODE

#if JTAG_ERASE_SRAM

    //Test USB to JTAG ERASE SRAM
    printf("ERASE SRAM TEST\r\n\r\n");
    rc=usb2jtag_Erase_Sram(devh,uiTimeout);
    if(rc != SUCCESS)
    {
        goto out;
    }

#endif // JTAG_ERASE_SRAM

#if JTAG_CONFIG_SRAM

    //Test USB to JTAG CONFIG SRAM
    printf("CONFIG SRAM TEST\r\n\r\n");
	rc = usb2jtag_Config_Sram(devh,filename,uiTimeout);
	while(length-2){
    length=usb2jtag_read_back(devh,500);
    }
	if(rc != SUCCESS)
    {
        goto out;
    }

#endif // JTAG_CONFIG_SRAM

#if JTAG_ERASE_T_FLASH

    //Test USB to JTAG ERASE T FLASH
    printf("ERASE T FLASH TEST\r\n\r\n");
    usb2jtag_Erase_T_Flash(devh,uiTimeout);
    while(length-2){
    length=usb2jtag_read_back(devh,500);
    }
    if(rc != SUCCESS)
    {
        goto out;
    }

#endif //JTAG_ERASE_T_FLASH

#if JTAG_PROGRAM_T_FLASH

    //Test USB to JTAG PROGRAM T FLASH
    printf("PROGRAM T FLASH TEST\r\n\r\n");
    usb2jtag_TckSet_Delay(devh,Select_Tck, uiTimeout);
    usb2jtag_embed_Program_Flash(devh,filename,0 ,uiTimeout);
    while(length-2){
    length=usb2jtag_read_back(devh,500);
    }
    if(rc != SUCCESS)
    {
        goto out;
    }

#endif //JTAG_PROGRAM_T_FLASH

#if JTAG_READ_T_FLASH

    //Test USB to JTAG READ T FLASH
    printf("READ T FLASH TEST\r\n\r\n");
    rc=usb2jtag_read_emb_Flash(devh,filename,uiTimeout);
    if(rc != SUCCESS)
    {
        goto out;
    }

#endif // JTAG_READ_T_FLASH


#if JTAG_ERASE_SPI_FLASH

    //Test USB to JTAG ERASE SPI FLASH
    printf("ERASE SPI FLASH TEST\r\n\r\n");
    rc=usb2jtag_Erase_SPI_Flash(devh,uiTimeout);
    if(rc != SUCCESS)
    {
        goto out;
    }

#endif // JTAG_ERASE_SPI_FLASH


#if JTAG_READ_SPI_FLASH_ID
out:
	if(devh)
    {
		libusb_close(devh);
    }

	libusb_exit(NULL);

    //Test USB to JTAG READ SPI FLASH ID
    printf("READ SPI FLASH ID TEST\r\n\r\n");
    rc=usb2jtag_read_SPI_Flash_id(devh,uiTimeout);
    if(rc != SUCCESS)
    {
        goto out;
    }

#endif // JTAG_READ_SPI_ID

#if JTAG_READ_SPI_FLASH

    //Test USB to JTAG READ SPI FLASH
    printf("READ SPI FLASH TEST\r\n\r\n");
    usb2jtag_read_SPI_Flash(devh,filename,uiTimeout);

#endif // JTAG_READ_SPI_FLASH

#if JTAG_PROGRAM_SPI_FLASH

    //Test USB to JTAG PROGRAM SPI FLASH
    printf("PROGRAM SPI FLASH TEST\r\n\r\n");
    rc=usb2jtag_Extern_Program_Flash(devh,filename,uiTimeout);
    if(rc != SUCCESS)
    {
        goto out;
    }
#endif // JTAG_PROGRAM_SPI_FLASH

out:
	libusb_release_interface(devh, Interface_number);

    return rc;
}

#endif // USB2JTAG_TEST

// ---------------------------------------- USB to JTAG ----------------------------------------//

//main
int main(int argc, char *argv[])
{
    int rc = 0;

	//Initializes libusb
	rc = libusb_init(NULL);
	if (rc != LIBUSB_SUCCESS)
    {
		return rc;
    }

    //Open USB device
	devh = libusb_open_device_with_vid_pid(NULL, VENDOR_ID, PRODUCT_ID);
	if(NULL == devh)
	{
		printf("Open USB device failed\n");
		goto out;
	}
	else
    {
        printf("Open USB device\n");
    }

#if USB2IIC_TEST
	rc = usb2iic_test(devh, pusb2iic_config);;
#endif // USB2IIC_TEST

#if USB2SPI_TEST
    rc = usb2spi_test(devh, pusb2spi_config);
#endif // USB2SPI_TEST

#if USB2UART_TEST
    usb2uart_test();
#endif // USB2UART_TEST

#if  USB2UART_VCP_TEST
    usb2uart_vcp_test();
#endif // USB2UART_VCP_TEST

#if USB2JTAG_TEST
    rc=usb2jtag_test(devh,500);
#endif // USB2JTAG_TEST


	if(rc != SUCCESS)
    {
        goto out;
    }

out:
	if(devh)
    {
		libusb_close(devh);
    }

	libusb_exit(NULL);

	return rc;
}
